%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "cpmsocket.h"
#include "cpmsocketlist.h"

/* Default error function */
#ifdef __cplusplus
extern "C" {
#endif

extern int yylex();
int addrerror(const char *msg);

#ifdef __cplusplus
}
#endif

/**
 * Global variable, defined in th lex file, used to count the number of lines
 * read.
 */
extern int lines;

PMSocketList *socketList;
PMSocket *protocolAddress;

typedef struct Ip_Port {
	char *ip;
	short port;
	transport trans;
} Ip_Port;

%}

%union {
	char *string;
	short number;
	struct Ip_Port *ip_port;
	struct PMSocketList *list;
	struct PMSocket *sock;
	enum transport trans;
}

%token<string> tPROTOCOL_ADDR
%token<string> tCONST_ADDR
%token<string> tCONST_ADDR_NAME
%token<string> tIP
%token<string> tIP_NUMBER
%token<string> tHOST
%token<string> tPORT
%token<number> tPORT_NUMBER
%token<number> tTRANS
%token<number> tTCP
%token<number> tUDP

%type<string>	Start
%type<sock>		ProtocolAddress
%type<ip_port>	Address
%type<sock>		RealConstAddress
%type<list>		ConstAddress
%type<trans>	TcpUdp
%type<ip_port>	IpPort
%type<string>	IpHost;

%%
Start	: ProtocolAddress ConstAddress
			{
				protocolAddress = $1;
				socketList = $2;
			}
		| error { addrerror("Erro na leitura do arquivo");
				  return -1;}
		;

ProtocolAddress	: tPROTOCOL_ADDR Address
					{
						PMSocket *sock = pmsocket_new($2->ip, $2->port, $2->trans);

						if (sock == NULL ||
							pmsocket_bind(sock) == PMSOCKET_ERROR) {
							char msg[1024];

							sprintf(msg, "%s  um endereo invlido!", $1);
							addrerror(msg);
							return -1;
						}

						sock->var = 0;
						sock->connected = 1;

						$$ = sock;
					}
				;

Address	: IpPort
			{
				$$ = $1;
			}

		| Address IpPort
			{
				if ($1->ip != NULL && $2->ip != NULL) {
					addrerror("Ip j foi definido");
					return -1;
				}
				else if ($1->ip == NULL && $2->ip != NULL)
					$1->ip = $2->ip;

				if ($1->port != -1 && $2->port != -1) {
					addrerror("Port j foi definido");
					return -1;
				}
				else if ($1->port == -1 && $2->port != -1)
					$1->port = $2->port;

				if ($1->trans != INVALID && $2->trans != INVALID) {
					addrerror("Transport j foi definido");
					return -1;
				}
				else if ($1->trans == INVALID && $2->trans != INVALID)
					$1->trans = $2->trans;

				free($2);
				$$ = $1;
			}
		;

IpPort	: tPORT tPORT_NUMBER
			{
				Ip_Port *ip_port = (Ip_Port *) malloc(sizeof(Ip_Port));

				ip_port->ip = NULL;
				ip_port->port = $2;
				ip_port->trans = INVALID;

				$$ = ip_port;
			}

		| tIP IpHost
			{
				Ip_Port *ip_port = (Ip_Port *) malloc(sizeof(Ip_Port));

				ip_port->ip = $2;
				ip_port->port = -1;
				ip_port->trans = INVALID;

				$$ = ip_port;
			}

		| tTRANS TcpUdp
			{
				Ip_Port *ip_port = (Ip_Port *) malloc(sizeof(Ip_Port));

				ip_port->ip = NULL;
				ip_port->port = -1;
				ip_port->trans = $2;

				$$ = ip_port;
			}

		| error { addrerror("Parmetro invlido! Apenas ip, porta e transporte podem ser especificado");
				  return -1; }
		;

TcpUdp	: tTCP { $$ = TCP; }
		| tUDP { $$ = UDP; }
		| error { addrerror("Protocolo de transport invlido");
				  return -1; }
		;

IpHost	: tIP_NUMBER { $$ = $1; }
		| tHOST { $$ = $1; }
		| error { addrerror("Tipo invlido! Deve ser um ip ou o nome de um host");
				  return -1; }
		;

ConstAddress	: RealConstAddress
					{
						PMSocketList *list = pmsocketlist_new();

						pmsocketlist_add(list, $1);
						$$ = list;
					}
				| ConstAddress RealConstAddress
					{
						pmsocketlist_add($1, $2);
						$$ = $1;
					}

				;

RealConstAddress	: tCONST_ADDR tCONST_ADDR_NAME Address
						{
							PMSocket *sock;

							fprintf(stderr, "Tentando conectar-se a %s%s..\n",
									$1, $2);
							sock = pmsocket_new($3->ip, $3->port, $3->trans);

							if (sock == NULL ||
								pmsocket_connect(sock) == PMSOCKET_ERROR) {
								char msg[1024];

								sprintf(msg, "%s%s  um endereo invlido!", $1, $2);
								addrerror(msg);
								return -1;
							}
							else fprintf(stderr, "Conectado com sucesso!\n\n");

							sock->var = 0;
							sock->connected = 1;

							$$ = sock;
						}

					;
%%

#ifdef __cpluscplus
extern "C" {
#endif

int addrerror(const char *msg) {
	fprintf(stderr, "Linha %d: %s!\n", lines+1, msg);

	return 0;
}

#ifdef __cplusplus
}
#endif