#include <stdlib.h>
#include "cpmqueue.h"

PMQueue *pmqueue_new(int qntMsgTypes) {
    PMQueue *queue = (PMQueue *) malloc(sizeof(PMQueue));
    int i;

    queue->qntMsgTypes = qntMsgTypes;
    queue->queues = (MessageList *) malloc(sizeof(MessageList) * qntMsgTypes);
    queue->rowMutex = (pmmutex_t *) malloc(sizeof(pmmutex_t) * qntMsgTypes);
    queue->newData = (pmcond_t *) malloc(sizeof(pmcond_t));
	queue->totalNewMutex = (pmmutex_t *) malloc(sizeof(pmmutex_t));
	queue->totalNew = 0;
    for (i = 0; i < qntMsgTypes; i++) {
        queue->queues[i].gm = NULL;
        queue->queues[i].next = NULL;
    }

    return queue;
}

void pmqueue_destroy(PMQueue **queue) {
    PMQueue *auxQueue = *queue;
    MessageList *ml, *aux;
    int i;

    pmcond_destroy(auxQueue->newData);
    for (i = 0; i < auxQueue->qntMsgTypes; i++) {
        pmmutex_destroy(&(auxQueue->rowMutex[i]));
        ml = &(auxQueue->queues[i]);
        while (ml->gm != NULL) {
            free(ml->gm);
            if (ml->next != NULL) {
                ml->gm = ml->next->gm;
                aux = ml->next;
                ml->next = ml->next->next;
                free(aux);
            }
            else ml->gm = NULL;
        }
    }

    free(*queue);
    (*queue) = NULL;
}

int pmqueue_enqueue(PMQueue *queue, GenericMessage *message) {
    int ret = CPM_QUEUE_OK;
	MessageList *ml;

    if (message == NULL || queue == NULL)
        ret = CPM_QUEUE_ERR;
    else if (message->code >= 0 && message->code < queue->qntMsgTypes) {
        pmmutex_lock(&(queue->rowMutex[message->code]));
        ml = &(queue->queues[message->code]);

        if (ml->gm == NULL)
            ml->gm = message;
        else {
            for (; ml->next != NULL; ml = ml->next);
            ml->next = (MessageList *) malloc(sizeof(MessageList));
            ml->next->gm = message;
            ml->next->next = NULL;
        }

		pmmutex_lock(queue->totalNewMutex);
		queue->totalNew++;
		pmmutex_unlock(queue->totalNewMutex);

        pmmutex_unlock(&(queue->rowMutex[message->code]));
    }
    else ret = CPM_QUEUE_ERR;

    if (ret == CPM_QUEUE_OK)
        pmcond_broadcast(queue->newData);

    return ret;
}

GenericMessage *pmqueue_dequeue(PMQueue *queue, unsigned totalCodes, int *codes, PMSocket *sock) {
    GenericMessage *msg = NULL;
    int finished = 0;
    int i;
    int ret;

    if (queue == NULL)
        return NULL;

	pmmutex_lock(queue->totalNewMutex);
	if (queue->totalNew == 0)
		pmcond_wait(queue->newData, queue->totalNewMutex);
	pmmutex_unlock(queue->totalNewMutex);

    for (i = 0; msg == NULL && finished < totalCodes; i = (i + 1) % totalCodes) {
        if (codes[i] >= 0 && codes[i] < queue->qntMsgTypes) {
            if (totalCodes - finished == 1)
                ret = pmmutex_lock(&(queue->rowMutex[codes[i]]));
            else
                ret = pmmutex_trylock(&(queue->rowMutex[codes[i]]));

            if (ret != PMTHREAD_ERROR) {
                MessageList *ml = &(queue->queues[codes[i]]);
                MessageList *aux = ml;
                int end = 0;

                while (ml->gm != NULL && !end) {
                    if (sock == NULL || PMSOCKET_EQUAL(ml->gm->sock, sock)) {
/*                        MessageList *aux;*/
                        msg = ml->gm;
                        if (aux != ml) {
                            aux->next = ml->next;
                            ml->next = NULL;
                            free(ml);
                        }
                        else {
                            if (ml->next == NULL)
                                ml->gm = NULL;
                            else {
                                ml->gm = ml->next->gm;
                                aux = ml->next;
                                ml->next = ml->next->next;
                                free(aux);
                            }
                        }
                        end++;
                    }
                    else {
                        if (aux != ml)
                            aux = aux->next;
                        ml = ml->next;
                        if (ml == NULL)
                            end++;
                    }
                }

                codes[i] = -1;
                finished++;
                pmmutex_unlock(&(queue->rowMutex[codes[i]]));

				pmmutex_lock(queue->totalNewMutex);
					if (msg != NULL)
						queue->totalNew--;
				pmmutex_unlock(queue->totalNewMutex);
            }
        }
    }

    return msg;
}
