#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include "cpmsocket.h"

PMSocket *pmsocket_new(const char *address, uint16_t port, transport trans)
{
    PMSocket *pmSocket = (PMSocket *) malloc(sizeof(PMSocket));
    struct hostent *hostEnt;

    if ((hostEnt = gethostbyname(address)) == NULL) {
		free(pmSocket);
        return NULL;
	}

    if (trans == TCP) {
        if ((pmSocket->socketFD = socket(PF_INET, SOCK_STREAM, 0)) == PMSOCKET_ERROR) {
			free(pmSocket);
        	return NULL;
		}
    }

	else if (trans == UDP) {
        if ((pmSocket->socketFD = socket(PF_INET, SOCK_DGRAM, 0)) == PMSOCKET_ERROR) {
			free(pmSocket);
        	return NULL;
		}
    }

    pmSocket->addr.sin_family = PF_INET;
    pmSocket->addr.sin_port = htons(port);
    memcpy(&(pmSocket->addr.sin_addr.s_addr), hostEnt->h_addr_list[0],
           hostEnt->h_length);
    pmSocket->trans = trans;

    return pmSocket;
}

int pmsocket_connect(PMSocket *pmSocket) {
	uint32_t addrLen = sizeof(pmSocket->addr);
	int result = connect(pmSocket->socketFD, (struct sockaddr *) &(pmSocket->addr), addrLen);

	if(result != PMSOCKET_ERROR)
		pmSocket->connected = 1;

	return result;
}

int pmsocket_bind(PMSocket *pmSocket) {
	uint32_t addrLen = sizeof(pmSocket->addr);
	int result = bind(pmSocket->socketFD, (struct sockaddr *) &(pmSocket->addr), addrLen);

	return result;
}

PMSocket *pmsocket_accept(PMSocket *pmSocket, int queue)
{
    PMSocket *client = (PMSocket *) malloc(sizeof(PMSocket));
    uint32_t addrLen;

    listen(pmSocket->socketFD, queue);
    client->trans = TCP;
    client->socketFD = accept(pmSocket->socketFD, (struct sockaddr *) &(pmSocket->addr), &addrLen);
	client->connected = 1;

    return client;
}

int pmsocket_close(PMSocket *pmSocket)
{
    if (pmSocket->socketFD != PMSOCKET_ERROR)
        close(pmSocket->socketFD);

    free(pmSocket);
    pmSocket = NULL;

    return 0;
}

int pmsocket_send(const PMSocket *pmSocket, const byte *data, uint32_t length)
{
    uint32_t sent;

    if (pmSocket->trans == TCP) {
        do {
            sent = send(pmSocket->socketFD, data, length, 0);
            length -= sent;
        } while (sent != PMSOCKET_ERROR && length > 0);
    }
    else if (pmSocket->trans == UDP) {
        do {
            sent = sendto(pmSocket->socketFD, data, length, 0,
                          (struct sockaddr *) &(pmSocket->addr),
                          (socklen_t) sizeof(pmSocket->addr));
            length -= sent;
        } while (sent != PMSOCKET_ERROR && length > 0);
    }

    return 0;
}

PMSocket *pmsocket_recv(const PMSocket *pmSocket, byte *data, uint32_t length, PMSocket *sender)
{
    uint32_t received;
    PMSocket *pms = (sender == NULL) ? (PMSocket *) malloc(sizeof(PMSocket)) : sender;

    if (pmSocket->trans == TCP) {
        do {
            received = recv(pmSocket->socketFD, data, length, 0);
            length -= received;
        } while (received != PMSOCKET_ERROR && length > 0);
        if (sender != NULL) {
            free(pms);
            pms = NULL;
        }

    }
    else if (pmSocket->trans == UDP) {
        uint32_t addrLen;
        do {
            received = recvfrom(pmSocket->socketFD, data, length, 0,
                                (struct sockaddr *) &(pms->addr),
                                  (socklen_t *) &addrLen);
            length -= received;
        } while (received != PMSOCKET_ERROR && length > 0);

        if (sender != NULL)
            pms = NULL;
    }

    return pms;
}

