#include <stdlib.h>
#include "cpmsocketlist.h"

static int calc_maxfd(PMSocketList *pmsl) {
	int maxfd = 0;
	pmsocketlist_iterator *iter = NULL;
	if(pmsl) {
		iter = pmsl->head;
		while(iter != NULL) {
			if(maxfd < iter->socket->socketFD) {
				maxfd = iter->socket->socketFD;
			}
			iter = iter->next;
		}
	}
	return maxfd;
}

PMSocketList *pmsocketlist_new() {
	PMSocketList *retorno = (PMSocketList *)malloc(sizeof(PMSocketList));
	if(retorno) {
		retorno->head = NULL;
		retorno->maxfd = 0;
		retorno->mutex = pmmutex_new();
	}
	return retorno;
}

void pmsocketlist_destroy(PMSocketList *pmsl) {
	pmsocketlist_iterator *iter = NULL;

	if(pmsl) {
		pmmutex_t mutex = pmsl->mutex;
		pmsocketlist_iterator *prev;

		pmmutex_lock(&mutex);
		iter = pmsl->head;
		prev = iter;
		while(iter != NULL) {
			pmsocket_close(iter->socket);
			free(iter->socket);
			iter = iter->next;
			free(prev);
			prev = iter;
		}
		free(pmsl);
		pmmutex_unlock(&mutex);
	}
}

int pmsocketlist_remove(PMSocketList *pmsl, PMSocket *sock) {
	pmsocketlist_iterator *iter = NULL;
	pmsocketlist_iterator *prev = NULL;
	int retorno = 0;

	pmmutex_lock(&pmsl->mutex);
	if(pmsl) {
		iter = pmsl->head;
		while(iter != NULL && iter->socket != sock) {
			prev = iter;
			iter = iter->next;
		}

		if(iter) {
			if(prev) {
				prev->next = iter->next;
			} else {
				pmsl->head = iter->next;
			}

			if(iter->socket->socketFD == pmsl->maxfd) {
				pmsl->maxfd = calc_maxfd(pmsl);
			}

			free(iter);
		} else {
			retorno = -1;
		}
	}
	pmmutex_unlock(&pmsl->mutex);
	return retorno;
}

int pmsocketlist_add(PMSocketList *pmsl, PMSocket *sock) {
	int retorno = 0;
	PMSocketListNode *node = NULL;

	pmmutex_lock(&pmsl->mutex);
	if(pmsl) {
		node = (PMSocketListNode*)malloc(sizeof(PMSocketListNode));
		node->socket = sock;
		node->next = pmsl->head;
		pmsl->head = node;

		if(sock->socketFD > pmsl->maxfd) {
			pmsl->maxfd = sock->socketFD;
		}
	} else {
		retorno = -1;
	}
	pmmutex_unlock(&pmsl->mutex);

	return retorno;
}

void pmsocketlist_lock(PMSocketList *pmsl) {
	pmmutex_lock(&pmsl->mutex);
}

void pmsocketlist_unlock(PMSocketList *pmsl) {
	pmmutex_unlock(&pmsl->mutex);
}
