#include "cpmthread.h"

// Threads
pmthread_t pmthread_new(void *(*thread)(void *args), void *args)
{
    pmthread_t id;
    int ret = pthread_create(&id, NULL, thread, args);

    return ret ? PMTHREAD_ERROR : id;
}

int pmthread_destroy(pmthread_t threadID)
{
    int ret = pthread_cancel(threadID);

    return ret ? PMTHREAD_ERROR : 0;
}

int pmthread_wait(pmthread_t threadID)
{
    int ret = pthread_join(threadID, NULL);
    ret |= pthread_detach(threadID);

    return ret ? PMTHREAD_ERROR : 0;
}

// Mutexs
pmmutex_t pmmutex_new()
{
    pmmutex_t mutex;
    pthread_mutex_init(&mutex, NULL);

    return mutex;
}

int pmmutex_destroy(pmmutex_t *mutex)
{
    int ret = pthread_mutex_destroy(mutex);

    return ret ? PMTHREAD_ERROR : 0;
}

int pmmutex_lock(pmmutex_t *mutex)
{
    int ret = pthread_mutex_lock(mutex);

    return ret ? PMTHREAD_ERROR : 0;
}

int pmmutex_trylock(pmmutex_t *mutex)
{
    int ret = pthread_mutex_trylock(mutex);
    
    return ret ? PMTHREAD_ERROR : 0;
}

int pmmutex_unlock(pmmutex_t *mutex)
{
    int ret = pthread_mutex_unlock(mutex);

    return ret ? PMTHREAD_ERROR : 0;
}

// Conditional variables
pmcond_t pmcond_new()
{
    pmcond_t cond;
    pthread_cond_init(&cond, NULL);

    return cond;
}

int pmcond_destroy(pmcond_t *cond)
{
    int ret = pthread_cond_destroy(cond);

    return ret ? PMTHREAD_ERROR : 0;
}

int pmcond_wait(pmcond_t *cond, pmmutex_t *mutex)
{
    int ret = pthread_cond_wait(cond, mutex);

    return ret ? PMTHREAD_ERROR : 0;
}

int pmcond_signal(pmcond_t *cond)
{
    int ret = pthread_cond_signal(cond);

    return ret ? PMTHREAD_ERROR : 0;
}

int pmcond_broadcast(pmcond_t *cond)
{
    int ret = pthread_cond_broadcast(cond);

    return ret ? PMTHREAD_ERROR : 0;
}

