#ifndef BINARY_EXPRESSION_H
#define BINARY_EXPRESSION_H

#include <string>
#include "expression.h"

using namespace std;

class BinaryExpression : public Expression {
	private:
		Expression *leftExpression;
		Expression *rightExpression;
		string op;

	public:
		BinaryExpression(SrcPosition *position, Expression *expression1, Expression *expression2, string op)
				: Ast(position), Expression(position), leftExpression(expression1), rightExpression(expression2), op(op) {}

		BinaryExpression(const BinaryExpression& binaryExpression) : Ast(position), Expression(binaryExpression.position) {
			*(this) = binaryExpression;
		}

		virtual ~BinaryExpression() {
			delete this->leftExpression;
			delete this->rightExpression;
		}


		Expression *getLeftExpression() {
			return this->leftExpression;
		}

		Expression *getRightExpression() {
			return this->rightExpression;
		}

		string getOp() const {
			return this->op;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitBinaryExpression(this);
		}

		virtual Ast& clone() const {
			return *(new BinaryExpression(new SrcPosition(*(this->position)),
					 this->leftExpression ? dynamic_cast<Expression*>(&(this->leftExpression->clone())) : 0,
					 this->rightExpression ? dynamic_cast<Expression*>(&(this->rightExpression->clone())) : 0,
					 this->op));
		}
};

#endif
