#include "ccodegeneratorvisitor.h"
#include "protomake.h"
#include <vector>
#include <list>
#include <set>
#include <typeinfo>
#include <cstdio>
#include <cctype>
#include "type.h"
using namespace std;

void CCodeGeneratorVisitor::generateMessageFile() {
	this->hFile << "/**\n * Messages file generated by ProtoMake 0.9 \n";
	this->hFile << " * Copyright 2003 Fabio Guerra, Igor Canana, Rodrigo Arajo &\n";
	this->hFile << " * Thiago Souto Maior.\n *\n";
	this->hFile << " * To contact the authors:\n";
	this->hFile << " *\tFabio   -> fga@cin.ufpe.br\n";
	this->hFile << " *\tIgor    -> icc@cin.ufpe.br\n";
	this->hFile << " *\tRodrigo -> rca4@cin.ufpe.br\n";
	this->hFile << " *\tThiago  -> tsmcf@cin.ufpe.br\n";
	this->hFile << " */\n\n";

	this->hFile << "#ifndef _MESSAGES_H_\n";
	this->hFile << "#define _MESSAGES_H_\n\n";

	this->hFile << "#include <netinet/in.h>\n";
	this->hFile << "#include <stdint.h>\n\n";

	this->hFile << "#ifndef _PM_BYTE_TYPE_\n";
	this->hFile << "#define _PM_BYTE_TYPE_\n";
	this->hFile << "\ttypedef unsigned char byte;\n";
	this->hFile << "#endif\n\n";

	this->cFile << "#include <string.h>\n";
	this->cFile << "#include <stdlib.h>\n";
	this->cFile << "#include \"" << this->baseName << "_messages.h\"\n\n";
}

void CCodeGeneratorVisitor::generateGenericMessageHeader(VisitorReturn *v) {
	char *baseName_up = new char[strlen(this->baseName)];
	unsigned i;

	for (i = 0; i < strlen(this->baseName); i++)
		baseName_up[i] = toupper(this->baseName[i]);
	baseName_up[i] = 0;

	this->hFile << "#ifndef " << baseName_up << "_GENERIC_MESSAGE_H_\n";
	this->hFile << "#define " << baseName_up << "_GENERIC_MESSAGE_H_\n\n";
	this->hFile << "#include \"genericmessage.h\"\n";
	this->hFile << "#include \"" << this->baseName << "_messages.h\"\n\n";
	this->hFile << "typedef struct " << baseName_up[0] << &this->baseName[1] << "_GenericMessage {\n";
	this->hFile << "\t_GENERIC_MESSAGE_COMMON_\n";
	this->hFile << "\tunion {\n";

	for (i = 0; i < v->names.size(); i++)
		this->hFile << "\t\tmessage_" << v->names[i] << " *" << v->names[i] << ";\n";

	this->hFile << "\t} msg;\n";
	this->hFile << "} " << baseName_up[0] << &this->baseName[1] << "_GenericMessage;\n\n";
	this->hFile << "#endif\n";
	delete baseName_up;
}

void CCodeGeneratorVisitor::addBehaviors() {
	char *fName = new char[strlen(this->baseDir) + 1024];
	char *temp;
	string name;
	char *behavior = "";
	char *function = "";

	for (TypeCheckerVisitor::BehaviorTableType::const_iterator itr = this->behaviorTable.begin();
		itr != this->behaviorTable.end();
		itr++) {
		temp = strdup(itr->first.c_str());
		behavior = strtok(temp, ".");
		function = strtok(NULL, ".");

		if (itr->first != name) {
			if (name != "") {
				this->cFile.close();
				this->hFile.close();
			}
			name = itr->first;
			sprintf(fName, "%s/behavior_%s.c", this->baseDir, behavior);
			if (this->genBehaviorCode)
				this->cFile.open(fName, ios::app);

			fName[strlen(fName)-1] = 'h';
			this->hFile.open(fName, ios::app);
		}

		this->hFile << "void " << function << "(";
		this->cFile << "void " << function << "(";
		for (unsigned param = 1; param <= itr->second->size(); param++) {
			if (param != 1) {
				this->hFile << ", ";
				if (this->genBehaviorCode)
					this->cFile << ", ";
			}
			switch ((*itr->second)[param-1]) {
				case Type::INTEGER:
				case Type::BOOLEAN:
				case Type::RANGE:
					this->hFile << "int *param" << param;
					if (this->genBehaviorCode)
						this->cFile << "int *param" << param;
					break;

				case Type::STRING:
					this->hFile << "char *param" << param;
					if (this->genBehaviorCode)
						this->cFile << "char *param" << param;
					break;

				case Type::ADDRESS:
				case Type::CLIENT_ADDRESS:
				case Type::SERVER_ADDRESS:
					this->hFile << "PMSocket *param" << param;
					if (this->genBehaviorCode)
						this->cFile << "PMSocket *param" << param;
					break;

				default:
					this->hFile << "void *param" << param;
					if (this->genBehaviorCode)
						this->cFile << "void *param" << param;
					break;
			}
		}

		this->hFile <<  ");\n";
		if (this->genBehaviorCode)
			this->cFile << ") {\n}\n\n";
	}
	this->hFile << "\n#endif\n";
	if (this->genBehaviorCode)
		this->cFile.close();
	this->hFile.close();
	delete [] fName;
}

void CCodeGeneratorVisitor::generateIPFile(vector<string> &names) {
	char name[1024];
	ofstream ipFile;
	TypeCheckerVisitor::ProcessTableType &st =
		const_cast<TypeCheckerVisitor::ProcessTableType&>(this->symbolTable);

	sprintf(name, "%s/%s.ip", this->baseDir, this->baseName);
	ipFile.open(name);
	ipFile << "# This file was generated by Protomake. You must provide valid addresses for\n";
	ipFile << "# each address described in this file. The ip, port and trans parameteres can\n";
	ipFile << "# be declared in any order.\n\n";
	ipFile << "[ Protocol Address ]\n";
	ipFile << "ip = 0.0.0.0\n";
	ipFile << "port = 0\n";
	ipFile << "trans = tcp\n\n";


	for (unsigned i = 0; i < names.size(); i++) {
		if (i != 0)
			ipFile << "\n";

		ipFile << "[ Constant Address " << names[i] << " ]\n";
		ipFile << "ip = 0.0.0.0\n";
		ipFile << "port = 0\n";
		if (st["main"][names[i]]->getTypeCode() == Type::ADDRESS)
			ipFile << "trans = udp\n";
		else ipFile << "trans = tcp\n";
	}

	ipFile.close();
}

void CCodeGeneratorVisitor::initializeArrays(vector<ArrayType*> &ats,
											 vector<string> &arrayNames) {
	vector<int> dimQnt;
	unsigned i;
	unsigned largest = 0;
	Type *type;

	for (i = 0; i < ats.size(); i++) {
		unsigned dimensions = 0;
		type = ats[i];
		ArrayType *aux;

		do {
			aux = dynamic_cast<ArrayType*>(type);

			dimensions++;
			type = aux->getType();
		} while (typeid(*type) == typeid(ArrayType));
		if (dimensions > largest)
			largest = dimensions;

		if (type->getTypeCode() == Type::STRING)
			dimensions++;
		dimQnt.push_back(dimensions);
	}

	if (largest > 1) {
		this->cFile << "\tunsigned ";
		for (i = 0; i < largest - 1; i++) {
			if (i != 0)
				this->cFile << ", ";
			this->cFile << "i" << i + 1;
		}
		this->cFile << ";\n";
	}

	for (i = 0; i < ats.size(); i++) {
		vector<int> dimSizes;
		string id = arrayNames[i];
		string varType = "";
		int j;
		type = ats[i];

		do {
			dimSizes.push_back(dynamic_cast<ArrayType*>(type)->getSize());
			type = dynamic_cast<ArrayType*>(type)->getType();
		} while (typeid(*type) == typeid(ArrayType));

		int minusOne = type->getTypeCode() == Type::STRING ? 1 : 0;

		switch(type->getTypeCode()) {
			case Type::INTEGER:
			case Type::RANGE:
			case Type::BOOLEAN:
				varType = "int";
				break;

			case Type::STRING:
				varType = "char";
				break;

			default:
				break;
		}

		this->cFile << "\n\t" << id << " = (" << varType;
		for (j = 0; j < dimQnt[i]; j++)
			this->cFile << "*";

		this->cFile << ") calloc(sizeof(" << varType;
		for (j = 0; j < dimQnt[i]- 1; j++)
			this->cFile << "*";

		this->cFile << "), " << dimSizes[0] << ");\n";

		char tabs[20] = "";
		for (j = 1; j < dimQnt[i] - minusOne; j++) {

			sprintf(tabs + j - 1, "\t");

			this->cFile << tabs << "for (i" << j << " = 0; ";
			this->cFile << "i" << j << " < " << dimSizes[j-1] << "; i" << j << "++) {\n";

			this->cFile << tabs << "\t" << id;
			for (int k = 1; k <= j; k++)
				this->cFile << "[i" << k << "]";
			this->cFile << " = " << "(" << varType;

			for (int k = 0; k < dimQnt[j]-j; k++)
				this->cFile << "*";

			this->cFile << ") calloc(sizeof(" << varType;
			for (int k = 0; k < dimQnt[j]-(j+1); k++)
				this->cFile << "*";

			this->cFile << "), " << dimSizes[j] << ");\n";
		}
		while (strcmp(tabs, "")) {
			this->cFile << tabs << "}\n";
			tabs[strlen(tabs) - 1] = 0;
		}
	}
	this->cFile << "\n";
}

VisitorReturn *CCodeGeneratorVisitor::visitAction(Action *action) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitActStatement(ActStatement *actStatement) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitArrayReference(ArrayReference *arrayReference) {
    VisitorReturn *v = 0;
	return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitArrayType(ArrayType *arrayType) {
    VisitorReturn *v = 0;
	bool end = false;
	Type *type = arrayType->getType();

	while (!end) {
		switch(type->getTypeCode()) {
			case Type::INTEGER:
			case Type::RANGE:
			case Type::BOOLEAN:
				this->cFile << "int ";
				end = true;
				break;
			case Type::STRING:
				this->cFile << "char ";
				end = true;
				break;
			case Type::ARRAY:
				type = dynamic_cast<ArrayType*>(type)->getType();
				break;
			default:
				end = true;
				break;
		}
	}

	return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitAssignStatement(AssignStatement *assignStatement) {
    VisitorReturn *v = 0;

	this->isAssignment = true;
	v = assignStatement->getLeftSideList()->visit(*this);
	this->isAssignment = false;

	if (v->names.size() == 1) {
		for (unsigned i = 0; i < this->currentTabStop; i++)
			this->cFile << "\t";

		this->cFile << v->names[0];
		if (v->exprs.size() != 0) {
			for (int i = v->exprs.size()-1; i >= 0; i--) {
				this->cFile << "[";
				v->exprs[i]->visit(*this);
				this->cFile << "]";
			}
		}
		this->cFile << " = ";

		assignStatement->getExpressionList()->visit(*this);
		this->cFile << ";\n";
	}
	else if (v->names.size() > 1) {
		ExpressionList *expl = assignStatement->getExpressionList();
		list<ExpressionList*> expressions;
		bool end = false;
		unsigned i;
		char tabs[20];

		while (!end) {
			if (typeid(*expl) == typeid(SequentialExpressionList)) {
				expressions.push_front(expl);
				expl = dynamic_cast<SequentialExpressionList*>(expl)->getExpressionList();
			}
			else {
				expressions.push_back(expl);
				end = true;
			}
		}

		for (i = 0; i < this->currentTabStop; i++)
			sprintf(tabs + i, "\t");
		this->cFile << tabs << "{\n";
		tabs[i] = '\t';
		tabs[i+1] = 0;


		for (i = 0; i < v->names.size(); i++) {
			char temp[20];
			TypeCheckerVisitor::ProcessTableType &st =
				const_cast<TypeCheckerVisitor::ProcessTableType&>(this->symbolTable);
			Type *type = st[this->currentProcess][v->names[i]];
			switch (type->getTypeCode()) {
				case Type::INTEGER:
				case Type::BOOLEAN:
				case Type::RANGE:
					this->cFile << tabs << "int ";
					break;

				case Type::STRING:
					this->cFile << tabs << "char *";
					break;

				default:
					break;
			}
			sprintf(temp, "%s__t_", v->names[i].c_str());
			this->cFile << temp << " = " << v->names[i] << ";\n";
			this->temps[v->names[i]] = temp;
		}
		this->cFile << endl;

		this->isAssignment = this->isExpression = true;
		list<ExpressionList*>::iterator itr;
		for (i = v->names.size() - 1, itr = expressions.begin();
			itr != expressions.end();
			itr++, i--) {
			ExpressionList *exprl = *itr;

			this->cFile << tabs << v->names[i] << " = ";
			if (typeid(*exprl) == typeid(SequentialExpressionList))
				dynamic_cast<SequentialExpressionList*>(exprl)->getSimpleExpressionList()->getExpression()->visit(*this);
			else
				dynamic_cast<SimpleExpressionList*>(exprl)->getExpression()->visit(*this);
			this->cFile << ";\n";
		}
		this->isAssignment = this->isExpression = false;
		tabs[strlen(tabs) - 1] = 0;
		this->cFile << tabs << "}\n";
	}

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitAst(Ast *ast) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitBehavior(Behavior *behavior) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitBehaviorStatement(BehaviorStatement *behaviorStatement) {
    VisitorReturn *v = 0;
	string tabs;
	for(unsigned i=0;i<this->currentTabStop;i++) {
		tabs += "\t";
	}
	
	
	this->cFile << tabs << behaviorStatement->getFunction()->getName() << "(";
	behaviorStatement->getExpressionList()->visit(*this);
	this->cFile << ");\n";
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitBinaryExpression(BinaryExpression *binaryExpression) {
    VisitorReturn *v = 0;
	string op = binaryExpression->getOp();

	if(op == "&") op = "&&";
	else if (op == "|") op = "||";
	else if (op == "~") op = "!";
	else if (op == "<>") op = "!=";
	else if (op == "=") op = "==";

	this->cFile << "(";
	binaryExpression->getLeftExpression()->visit(*this);
	this->cFile << " " << op << " ";
	binaryExpression->getRightExpression()->visit(*this);
	this->cFile << ")";
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitDeclaration(Declaration *declaration) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitDoStatement(DoStatement *doStatement) {
    VisitorReturn *v = 0;
	SimpleGuardedStatement *sgs = doStatement->getSimpleGuardedStatement();
	char tabs[20];

	for (unsigned i = 0; i < this->currentTabStop; i++)
		sprintf(tabs + i, "\t");

	this->cFile << tabs << "while (";
	sgs->getExpression()->visit(*this);
	this->cFile << ") {\n";

	this->currentTabStop++;
	sgs->getStatement()->visit(*this);
	this->currentTabStop--;
	this->cFile << tabs << "}\n";

	return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitExpression(Expression *expression) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitExpressionList(ExpressionList *expressionList) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitFieldDeclaration(FieldDeclaration *fieldDeclaration) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitFieldReference(FieldReference *fieldReference) {
    VisitorReturn *v = 0;
	string name = fieldReference->getMsgId()->getName() + "->" +
				  fieldReference->getFieldId()->getName();

	if (this->isAssignment) {
		v = new VisitorReturn;
		v->names.push_back(name);

		if (this->isExpression) {
			if (this->temps[name] != "")
				this->cFile << this->temps[name];
			else this->cFile << name;
		}
	}
	else this->cFile << name;

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitFieldType(FieldType *fieldType) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitGuard(Guard *guard) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitGuardedStatement(GuardedStatement *guardedStatement) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitGuardExpression(GuardExpression *guardExpression) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitGuardReceive(GuardReceive *guardReceive) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitGuardTimeout(GuardTimeout *guardTimeout) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitIdentifier(Identifier *identifier) {
    VisitorReturn *v = 0;

	if (this->isAssignment) {
		v = new VisitorReturn;
		v->names.push_back(identifier->getName());

		if (this->isExpression) {
			if (this->temps[identifier->getName()] != "")
				this->cFile << this->temps[identifier->getName()];
			else this->cFile << identifier->getName();
		}
	}
	else this->cFile << identifier->getName();

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitIdentifierList(IdentifierList *identifierList) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitIfStatement(IfStatement *ifStatement) {
    VisitorReturn *v = 0;
	char tabs[20];

	v = ifStatement->getGuardedStatement()->visit(*this);

	for (unsigned i = 0; i < this->currentTabStop; i++)
		sprintf(tabs + i, "\t");

	this->cFile << tabs << "if (";
	for (vector<SimpleGuardedStatement*>::iterator itr = v->guardedStmts.begin();
		 itr != v->guardedStmts.end();
		 itr++) {
		SimpleGuardedStatement* sgs = *itr;

		if (itr != v->guardedStmts.begin())
			this->cFile << tabs << "else if (";

		sgs->getExpression()->visit(*this);
		this->cFile << ") {\n";
		this->currentTabStop++;
		sgs->getStatement()->visit(*this);
		this->currentTabStop--;
		this->cFile << tabs << "}\n";
	}
	delete v;
	v = 0;

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitImport(Import *import) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitIntegerLiteral(IntegerLiteral *integerLiteral) {
    VisitorReturn *v = 0;
	this->cFile << integerLiteral->getValue();
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitLeftSide(LeftSide *leftSide) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitLeftSideList(LeftSideList *leftSideList) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitMessage(Message *message) {
    VisitorReturn *v = 0;

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitMultipleArrayReference(MultipleArrayReference *multipleArrayReference) {
    VisitorReturn *v = 0;

	v = multipleArrayReference->getArrayReference()->visit(*this);
	if (this->isAssignment)
		v->exprs.push_back(multipleArrayReference->getExpression());
	else {
		this->cFile << "[";
		multipleArrayReference->getExpression()->visit(*this);
		this->cFile << "]";
	}

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitProcess(Process *process) {
    VisitorReturn *v = 0;
    return v;
}

// Terminado!
VisitorReturn *CCodeGeneratorVisitor::visitProgram(Program *program) {
    VisitorReturn *v = 0;
    char *name = new char[strlen(this->baseName) + strlen(this->baseDir) + 256];

    if(program->getImport())
        program->getImport()->visit(*this); // No est sendo considerado ainda

    // Creating messages files!
    sprintf(name, "%s/%s_messages.h", baseDir, baseName);
    this->hFile.open(name);

    name[strlen(name) - 1] = 'c';
    this->cFile.open(name);

    if (this->hFile.is_open() && this->cFile.is_open())
		this->generateMessageFile();
    else {
        cerr << "Unable to create messages.h and messages.c!\n";
        exit(1);
    }
    v = program->getMessage()->visit(*this);
	this->hFile << "#endif\n";
	this->hFile.close();
	this->cFile.close();

	// Generate the generic message header for this protocol
    sprintf(name, "%s/%s_genericmessage.h", this->baseDir, this->baseName);
    this->hFile.open(name);
	this->generateGenericMessageHeader(v);
	this->hFile.close();

	// Generate behaviors
    if(program->getBehavior())
        program->getBehavior()->visit(*this);

	sprintf(name, "%s/%s_process.h", baseDir, baseName);
    this->hFile.open(name);
	name[strlen(name) - 1] = 'c';
	this->cFile.open(name);

	this->hFile << "#ifndef _";
	for (unsigned i = 0; i < strlen(baseName); i++)
		this->hFile << char(toupper(baseName[i]));
	this->hFile << "_PROCESS_H_\n";

	this->hFile << "#define _";
	for (unsigned i = 0; i < strlen(baseName); i++)
		this->hFile << char(toupper(baseName[i]));
	this->hFile << "_PROCESS_H_\n\n";

	this->cFile << "#include <stdlib.h>\n";
	this->cFile << "#include \"" << baseName << "_process.h\"\n";
	this->cFile << "#include \"" << baseName << "_genericmessage.h\"\n";
	this->cFile << "#include \"cpmqueue.h\"\n\n";
	this->cFile << "extern PMQueue *messageQueue;\n";

	// Generate process
    v = program->getProcess()->visit(*this);

	this->hFile << "#endif\n";
	this->hFile.close();
	this->cFile.close();

	// Adding behaviors
	this->addBehaviors();

    delete [] name;

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitRangeType(RangeType *rangeType) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitReceiveStatement(ReceiveStatement *receiveStatement) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSendStatement(SendStatement *sendStatement) {
    VisitorReturn *v = 0;
	string tabs;
	
	for(unsigned i=0;i<this->currentTabStop;i++) {
		tabs += "\t";
	}
	
	this->cFile << tabs << "{\n" << tabs << "\tint message_" << sendStatement->getMsgId()->getName() << "_length;\n" 
				<< tabs << "\tbyte *buf = message_" << sendStatement->getMsgId()->getName() << "_marshall(" 
				<< sendStatement->getMsgId()->getName() << ", NULL, &message_" << sendStatement->getMsgId()->getName() 
				<< "_length);\n" << tabs << "\tif(pmsocket_send(" << sendStatement->getAddrId()->getName()
			    << ", buf, message_" << sendStatement->getMsgId()->getName() << "_length) != 0) {\n"
				<< tabs << "\t\tprintf(\"bugou!\\n\");\n" << tabs << "\t\texit(1);\n" << tabs <<"\t}\n";
	
	this->cFile << tabs << "}\n";
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialAction(SequentialAction *sequentialAction) {
    VisitorReturn *v1, *v2;

    // Incompleto!!!
    v1 = sequentialAction->getAction()->visit(*this);
    v2 = sequentialAction->getSimpleAction()->visit(*this);

    if (!v1)
        v1 = v2;

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialBehavior(SequentialBehavior *sequentialBehavior) {
    VisitorReturn *v = 0;

    sequentialBehavior->getBehavior()->visit(*this);
    sequentialBehavior->getSimpleBehavior()->visit(*this);

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialDeclaration(SequentialDeclaration *sequentialDeclaration) {
    VisitorReturn *v1, *v2;

    v1 = sequentialDeclaration->getDeclaration()->visit(*this);
    v2 = sequentialDeclaration->getSimpleDeclaration()->visit(*this);

	for (int i = v2->names.size() - 1; i >= 0; i--)
		v1->names.push_back(v2->names[i]);

	for (unsigned i = 0; i < v2->ats.size(); i++)
		v1->ats.push_back(v2->ats[i]);

	for (unsigned i = 0; i < v2->arrayNames.size(); i++)
		v1->arrayNames.push_back(v2->arrayNames[i]);

	delete v2;

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialExpressionList(SequentialExpressionList *sequentialExpressionList) {
    VisitorReturn *v = 0;
	sequentialExpressionList->getExpressionList()->visit(*this);
	sequentialExpressionList->getSimpleExpressionList()->visit(*this);
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialFieldDeclaration(SequentialFieldDeclaration *sequentialFieldDeclaration) {
    VisitorReturn *v1, *v2;

    v1 = sequentialFieldDeclaration->getFieldDeclaration()->visit(*this);
    v2 = sequentialFieldDeclaration->getSimpleFieldDeclaration()->visit(*this);

    v1->fields.push_back(v2->fields[0]);

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialGuardedStatement(SequentialGuardedStatement *sequentialGuardedStatement) {
    VisitorReturn *v;

	v = sequentialGuardedStatement->getGuardedStatement()->visit(*this);
	v->guardedStmts.push_back(sequentialGuardedStatement->getSimpleGuardedStatement());

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialIdentifierList(SequentialIdentifierList *sequentialIdentifierList) {
    VisitorReturn *v1, *v2;

	v1 = sequentialIdentifierList->getIdentifierList()->visit(*this);
	this->cFile << ", ";
    v2 = sequentialIdentifierList->getSimpleIdentifierList()->visit(*this);

	if (v1 == 0)
		v1 = v2;
	else if (v2 != 0) {
		for (unsigned i = 0; i < v2->arrayNames.size(); i++)
			v1->arrayNames.push_back(v2->arrayNames[i]);
		delete v2;
	}

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialImport(SequentialImport *sequentialImport) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialLeftSideList(SequentialLeftSideList *sequentialLeftSideList) {
    VisitorReturn *v1, *v2;

	v1 = sequentialLeftSideList->getLeftSideList()->visit(*this);
	v2 = sequentialLeftSideList->getSimpleLeftSideList()->visit(*this);

	if (v1 == 0)
		v1 = v2;
	else if (v2 != 0) {
		for (unsigned i = 0; i < v2->names.size(); i++)
			v1->names.push_back(v2->names[i]);
		delete v2;
	}

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialMessage(SequentialMessage *sequentialMessage) {
    VisitorReturn *v1 = 0, *v2 = 0;

    v1 = sequentialMessage->getMessage()->visit(*this);
    v2 = sequentialMessage->getSimpleMessage()->visit(*this);

	v1->names.push_back(v2->names[0]);
	delete v2;

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialProcess(SequentialProcess *sequentialProcess) {
    VisitorReturn *v1, *v2;

    v1 = sequentialProcess->getProcess()->visit(*this);
    v2 = sequentialProcess->getSimpleProcess()->visit(*this);

    if (!v1)
        v1 = v2;

    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSequentialStatement(SequentialStatement *sequentialStatement) {
    VisitorReturn *v1, *v2;

    // Incompleto!!!
    v1 = sequentialStatement->getStatement()->visit(*this);
    v2 = sequentialStatement->getSimpleStatement()->visit(*this);

    if (!v1)
        v1 = v2;
    return v1;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleAction(SimpleAction *simpleAction) {
    VisitorReturn *v = 0;

    // Incompleto!!!
    v = simpleAction->getStatement()->visit(*this);

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleArrayReference(SimpleArrayReference *simpleArrayReference) {
    VisitorReturn *v = 0;
	char tabs[20];

	if (this->isAssignment) {
		v = new VisitorReturn;
		v->names.push_back(simpleArrayReference->getIdentifier()->getName());
		v->exprs.push_back(simpleArrayReference->getExpression());
	}
	else {
		for (unsigned i = 0; i < this->currentTabStop; i++)
			sprintf(tabs + i, "\t");

		this->cFile << tabs << simpleArrayReference->getIdentifier()->getName();
		this->cFile << "[";
		simpleArrayReference->getExpression()->visit(*this);
		this->cFile << "]";
	}

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleBehavior(SimpleBehavior *simpleBehavior) {
    VisitorReturn *v = 0;
    IdentifierList *iList = simpleBehavior->getIdentifierList();
    string identifier;
    bool end = false;
    char *name = new char[strlen(this->baseDir) + 256];
    ofstream file;

    do {
        if (typeid(*iList) == typeid(SimpleIdentifierList)) {
            identifier = dynamic_cast<SimpleIdentifierList*>(iList)->getIdentifier()->getName();
            end = true;
          }
          else if (typeid(*iList) == typeid(SequentialIdentifierList)) {
            identifier = dynamic_cast<SequentialIdentifierList*>(iList)->getSimpleIdentifierList()->getIdentifier()->getName();
            iList = dynamic_cast<SequentialIdentifierList*>(iList)->getIdentifierList();
        }

        sprintf(name, "%s/behavior_%s.c", this->baseDir, identifier.c_str());
        if (this->genBehaviorCode) {
            file.open(name);
            file << "#include \"behavior_" << identifier << ".h\"\n\n";
            file.close();
        }

        name[strlen(name) - 1] = 'h';
        file.open(name);
        for (unsigned i = 0; i < identifier.length(); i++)
            identifier[i] = toupper(identifier[i]);
        file << "#ifndef _BEHAVIOR_" << identifier << "_H_\n";
        file << "#define _BEHAVIOR_" << identifier << "_H_\n\n";
        file.close();


    } while (!end);

    delete [] name;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleDeclaration(SimpleDeclaration *simpleDeclaration) {
    VisitorReturn *v = new VisitorReturn;

	if(!(simpleDeclaration->getType()->getTypeCode() == Type::ADDRESS ||
		simpleDeclaration->getType()->getTypeCode() == Type::CLIENT_ADDRESS ||
		simpleDeclaration->getType()->getTypeCode() == Type::SERVER_ADDRESS)) {

		this->cFile << "\t";
		if(simpleDeclaration->isConstant()) {
			this->cFile << "const ";
		}
		simpleDeclaration->getType()->visit(*this);

		Type *type = simpleDeclaration->getType();
		if (typeid(*type) == typeid(ArrayType)) {
			this->pilha.push_back(type);
		}

		this->pilha.push_back(simpleDeclaration);
		VisitorReturn *vTemp = simpleDeclaration->getIdentifierList()->visit(*this);
		this->pilha.pop_back();
		if (typeid(*type) == typeid(ArrayType))
			this->pilha.pop_back();

		if (vTemp != 0) {
			ArrayType *aux = dynamic_cast<ArrayType*>(type);

			for (unsigned i = 0; i < vTemp->arrayNames.size(); i++) {
				v->arrayNames.push_back(vTemp->arrayNames[i]);
				v->ats.push_back(aux);
			}
			delete vTemp;
		}

		this->cFile << ";\n";
	}

	if ((simpleDeclaration->getType()->getTypeCode() == Type::ADDRESS ||
		simpleDeclaration->getType()->getTypeCode() == Type::CLIENT_ADDRESS ||
		simpleDeclaration->getType()->getTypeCode() == Type::SERVER_ADDRESS) &&
		simpleDeclaration->isConstant()) {
		bool end = false;
		IdentifierList *aux = simpleDeclaration->getIdentifierList();

		do {
			if (typeid(*aux) == typeid(SimpleIdentifierList)) {
				SimpleIdentifierList *id =
					dynamic_cast<SimpleIdentifierList*>(aux);

				v->names.push_back(id->getIdentifier()->getName());
				end = true;
			}
			else {
				SequentialIdentifierList *id =
					dynamic_cast<SequentialIdentifierList*>(aux);
				string name =
					id->getSimpleIdentifierList()->getIdentifier()->getName();

				v->names.push_back(name);
				aux = id->getIdentifierList();
			}

		} while (!end);

	}

	return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleExpressionList(SimpleExpressionList *simpleExpressionList) {
    VisitorReturn *v = 0;
	simpleExpressionList->getExpression()->visit(*this);
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleFieldDeclaration(SimpleFieldDeclaration *simpleFieldDeclaration) {
    VisitorReturn *v = new VisitorReturn;
    FieldType::FType fType = simpleFieldDeclaration->getFieldType()->getType();
    v->fields.push_back(simpleFieldDeclaration);

    Expression *expr = simpleFieldDeclaration->getFieldType()->getExpression();

    this->hFile << "\t";
    if (fType == FieldType::BYTE || fType == FieldType::BIT) {
        if (typeid(*expr) == typeid(IntegerLiteral)) {
            if (fType == FieldType::BYTE)
                this->hFile << "byte *";

            else if (fType == FieldType::BIT) {
                IntegerLiteral *il = dynamic_cast<IntegerLiteral*> (expr);

                if (il->getValue() <= 8)
                    this->hFile << "byte ";
                else if (il->getValue() <= 16)
                    this->hFile << "uint16_t ";
                else if (il->getValue() <= 32)
                    this->hFile << "uint32_t ";
            }
        }
        else if (typeid(*expr) == typeid(Identifier))
            this->hFile << "byte *";
    }
    else this->hFile << "byte *";

    this->hFile << simpleFieldDeclaration->getIdentifier()->getName() << ";\n";

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleGuardedStatement(SimpleGuardedStatement *simpleGuardedStatement) {
    VisitorReturn *v = new VisitorReturn;

	v->guardedStmts.push_back(simpleGuardedStatement);

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleIdentifierList(SimpleIdentifierList *simpleIdentifierList) {
    VisitorReturn *v = 0;
	bool isSimpleDecl = false;
	bool isArrayType = false;
	SimpleDeclaration *sd = NULL;
	ArrayType *at = NULL;

	if (this->pilha.size() > 0) {
		if (typeid(*this->pilha[this->pilha.size()-1]) == typeid(SimpleDeclaration)) {
			isSimpleDecl = true;
			sd = dynamic_cast<SimpleDeclaration*>(this->pilha[this->pilha.size()-1]);
		}

		if (this->pilha.size() > 1 &&
			typeid(*this->pilha[this->pilha.size()-2]) == typeid(ArrayType)) {
			at = dynamic_cast<ArrayType*>(this->pilha[this->pilha.size()-2]);
			isArrayType = true;
		}
	}

	if(isSimpleDecl) {
		if (!isArrayType && sd->getType()->getTypeCode() == Type::STRING)
			this->cFile << "*";
		else if (isArrayType && !sd->isConstant()) {
			v = new VisitorReturn;
			v->arrayNames.push_back(simpleIdentifierList->getIdentifier()->getName());
			Type *type = sd->getType();

			do {
				this->cFile << "*";
				type = dynamic_cast<ArrayType*>(type)->getType();
			} while (typeid(*type) == typeid(ArrayType));
			if (type->getTypeCode() == Type::STRING)
				this->cFile << "*";
		}
	}

	this->cFile << simpleIdentifierList->getIdentifier()->getName();

	if(isSimpleDecl) {
		if (isArrayType && sd->isConstant()) {
			bool end = false;
			do {
				this->cFile << "[" << at->getSize() << "]";
				if (at->getType()->getTypeCode() != Type::ARRAY)
					end = true;
				else
					at = dynamic_cast<ArrayType*>(at->getType());
			} while (!end);
		}

		switch(sd->getType()->getTypeCode()) {
			case Type::INTEGER:
			case Type::RANGE:
			case Type::BOOLEAN:
				this->cFile << " = ";
				if (sd->getExpression())
					sd->getExpression()->visit(*this);
				else this->cFile << "0";
				break;

			case Type::ARRAY:
				if (isArrayType && !sd->isConstant())
					this->cFile << " = NULL";
				else this->cFile << "";
				break;

			case Type::STRING:
				this->cFile << " = ";
				if (sd->getExpression())
					sd->getExpression()->visit(*this);
				else this->cFile << "\"\"";
				break;

			default:
				break;
		}
	}

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleImport(SimpleImport *simpleImport) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleLeftSideList(SimpleLeftSideList *simpleLeftSideList) {
    VisitorReturn *v = 0;

	v = simpleLeftSideList->getLeftSide()->visit(*this);

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleMessage(SimpleMessage *simpleMessage) {
    VisitorReturn *v = 0;
    static int code = 1;

	string name = simpleMessage->getIdentifier()->getName();
	this->codeMap[name] = code;


// Generate correct header for the messages.c file!
    this->hFile << "typedef struct {\n";
    v = simpleMessage->getFieldDeclaration()->visit(*this);
    this->hFile << "} message_" << name << ";\n\n";

    this->hFile << "byte *message_" << name << "_marshall(const message_" << name;
    this->hFile << " *msg_" << name << ", byte *buffer, int *length);\n";

    this->hFile << "message_" << name << " *message_" << name << "_unmarshall(";
    this->hFile << "const byte *buffer, message_" << name << " *msg_" << name << ");\n\n";


// Generate the implementation for the prototypes declared in message.h!
// Serialize the struct
    this->cFile << "byte *message_" << name << "_marshall(const message_" << name;
    this->cFile << " *msg_" << name << ", byte *buffer, int *length) {\n";

    this->cFile << "\tuint16_t code = htons(" << code << ");\n";
    this->cFile << "\tuint32_t offset = 0;\n";
    this->cFile << "\tuint16_t temp16 = 0;\n";
    this->cFile << "\tuint32_t temp32 = 0;\n";
    this->cFile << "\tbyte *buf;\n\n";
	this->cFile << "\t*length = ";
    //this->cFile << "\tbuf = (byte *) malloc((";

// Allocate the correct size for the buffer
    for (vector<SimpleFieldDeclaration*>::iterator itr = v->fields.begin();
         itr != v->fields.end();
         itr++) {
        if (itr != v->fields.begin())
            this->cFile << " + ";

        Expression *expr = (*itr)->getFieldType()->getExpression();

        if (typeid(*expr) == typeid(IntegerLiteral)) {
            IntegerLiteral *il = dynamic_cast<IntegerLiteral*> (expr);

            if ((*itr)->getFieldType()->getType() == FieldType::BYTE)
                this->cFile << il->getValue();

            else if ((*itr)->getFieldType()->getType() == FieldType::BIT) {

                if (il->getValue() <= 8)
                    this->cFile << "1";
                else if (il->getValue() <= 16)
                    this->cFile << "2";
                else if (il->getValue() <= 32)
                    this->cFile << "4";
            }
        }
        else if (typeid(*expr) == typeid(Identifier))
            this->cFile << "msg_" << name << "->" << (dynamic_cast<Identifier*> (expr))->getName();
    }
	this->cFile << ";\n";
	this->cFile << "\tbuf = (byte *) malloc((*length) * sizeof(byte));\n\n";
    this->cFile << "\tmemcpy(buf, &code, 2);\n";
    this->cFile << "\toffset = 2;\n\n";

    for (vector<SimpleFieldDeclaration*>::iterator itr = v->fields.begin();
         itr != v->fields.end();
         itr++) {
        Expression *expr = (*itr)->getFieldType()->getExpression();

        if (typeid(*expr) == typeid(IntegerLiteral)) {
            IntegerLiteral *il = dynamic_cast<IntegerLiteral*> (expr);

            if ((*itr)->getFieldType()->getType() == FieldType::BYTE) {
                this->cFile << "\tmemcpy((buf + offset), ";
                this->cFile << "msg_" << name << "->" << (*itr)->getIdentifier()->getName();
                this->cFile << ", " << il->getValue() << ");\n";
                this->cFile << "\toffset += " << il->getValue() << ";\n\n";
            }

            else if ((*itr)->getFieldType()->getType() == FieldType::BIT) {
                if (il->getValue() > 8 && il->getValue() <= 16) {
                    this->cFile << "\ttemp16 = htons(msg_" << name << "->" << (*itr)->getIdentifier()->getName();
                    this->cFile << ");\n";
                }

                else if (il->getValue() > 16 && il->getValue() <= 32) {
                    this->cFile << "\ttemp32 = htonl(msg_" << name << "->" << (*itr)->getIdentifier()->getName();
                    this->cFile << ");\n";
                }

                this->cFile << "\tmemcpy((buf + offset), ";

                if (il->getValue() <= 8) {
                    this->cFile << "&(msg_" << name << "->" << (*itr)->getIdentifier()->getName() << "), ";
                    this->cFile << "1);\n\toffset += 1;\n\n";
                }
                else if (il->getValue() <= 16)
                    this->cFile << "&temp16, 2);\n\toffset += 2;\n\n";
                else if (il->getValue() <= 32)
                    this->cFile << "&temp32, 4);\n\toffset += 4;\n\n";
            }
        }
        else if (typeid(*expr) == typeid(Identifier)) {
            this->cFile << "\tmemcpy((buf + offset), ";
            this->cFile << "msg_" << name << "->" << (*itr)->getIdentifier()->getName() << ", ";
            this->cFile << "msg_" << name << "->" << (dynamic_cast<Identifier*> (expr))->getName() << ");\n";
            this->cFile << "\toffset += msg_" << name << "->";
            this->cFile << (dynamic_cast<Identifier*> (expr))->getName() << ";\n\n";
        }
    }
    this->cFile << "\tif (buffer != NULL) {\n";
    this->cFile << "\t\tmemmove(buffer, buf, offset);\n";
    this->cFile << "\t\tfree(buf);\n\t\tbuf = NULL;\n\t}\n\n";
    this->cFile << "\treturn buf;\n}\n\n";


// Unserialize the buffer
    this->cFile << "message_" << name << " *message_" << name << "_unmarshall(";
    this->cFile << "const byte *buffer, message_" << name << " *msg_" << name << ") {\n";

    this->cFile << "\tmessage_" << name << " *msg = (message_" << name << " *) ";
    this->cFile << "malloc(sizeof(message_" << name << "));\n";
    this->cFile << "\tuint32_t offset = 2;\n\n";

    for (vector<SimpleFieldDeclaration*>::iterator itr = v->fields.begin();
         itr != v->fields.end();
         itr++) {
        Expression *expr = (*itr)->getFieldType()->getExpression();

        if (typeid(*expr) == typeid(IntegerLiteral)) {
            IntegerLiteral *il = dynamic_cast<IntegerLiteral*> (expr);

            if ((*itr)->getFieldType()->getType() == FieldType::BYTE) {
                this->cFile << "\tmemcpy(msg_" << name << "->" << (*itr)->getIdentifier()->getName();
                this->cFile << ", (buffer + offset), " << il->getValue() << ");\n";
                this->cFile << "\toffset += " << il->getValue() << ";\n\n";
            }

            else if ((*itr)->getFieldType()->getType() == FieldType::BIT) {
                this->cFile << "\tmemcpy(&(msg_" << name << "->" << (*itr)->getIdentifier()->getName();
                this->cFile << "), (buffer + offset), ";

                if (il->getValue() <= 8)
                    this->cFile << "1);\n\toffset += 1;\n\n";
                else if (il->getValue() <= 16) {
                    this->cFile << "2);\n\toffset += 2;\n";
                    this->cFile << "\tmsg_" << name << "->" << (*itr)->getIdentifier()->getName() << " = ";
                    this->cFile << "ntohs(msg_" << name << "->" << (*itr)->getIdentifier()->getName() << ");\n\n";
                }
                else if (il->getValue() <= 32) {
                    this->cFile << "4);\n\toffset += 4;\n";
                    this->cFile << "\tmsg_" << name << "->" << (*itr)->getIdentifier()->getName() << " = ";
                    this->cFile << "ntohl(msg_" << name << "->" << (*itr)->getIdentifier()->getName() << ");\n\n";
                }
            }
        }
        else if (typeid(*expr) == typeid(Identifier)) {
            this->cFile << "\tmemcpy(msg_" << name << "->" << (*itr)->getIdentifier()->getName();
            this->cFile << ", (buffer + offset), ";
            this->cFile << "msg_" << name << "->" << (dynamic_cast<Identifier*> (expr))->getName() << ");\n";
            this->cFile << "\toffset += msg_" << name << "->";
            this->cFile << (dynamic_cast<Identifier*> (expr))->getName() << ";\n\n";
        }
    }

    this->cFile << "\tif (msg_" << name << " != NULL) {\n";
    this->cFile << "\t\tmemmove(msg_" << name << ", msg, offset);\n";
    this->cFile << "\t\tfree(msg);\n\t\tmsg = NULL;\n\t}\n\n";
    this->cFile << "\treturn msg;\n}\n\n";

// Increment the code for the next message
    code++;
    delete v;
    v = 0;

// Return the name of the message
	v = new VisitorReturn;
	v->names.push_back(name);

    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleProcess(SimpleProcess *simpleProcess) {
    VisitorReturn *v = 0;

	this->currentProcess = simpleProcess->getIdentifier()->getName();
	this->hFile << "void thread_process_" << simpleProcess->getIdentifier()->getName() << "(void *param);\n\n";
	this->cFile << "void thread_process_" << simpleProcess->getIdentifier()->getName() << "(void *param) {\n";

	v = simpleProcess->getDeclaration()->visit(*this);
	if (v) this->initializeArrays(v->ats, v->arrayNames);


	if (simpleProcess->isMainProcess())
		this->generateIPFile(v->names);
	v->names.clear();

	Action *action = simpleProcess->getAction();
	vector<SimpleAction*> expressions;
	vector<SimpleAction*> receives;
	vector<SimpleAction*> timeouts;

	//Preenche os vectors de Guardas;
	bool continua = true;
	while(continua) {
		if(typeid(*action) == typeid(SimpleAction)) {
			SimpleAction *simpleAction = dynamic_cast<SimpleAction*>(action);
			if(typeid(*(simpleAction->getGuard())) == typeid(GuardExpression)) {
				expressions.push_back(simpleAction);
			} else if(typeid(*(simpleAction->getGuard())) == typeid(GuardReceive)) {
				receives.push_back(simpleAction);
			} else if(typeid(*(simpleAction->getGuard())) == typeid(GuardTimeout)) {
				timeouts.push_back(simpleAction);
			}
			continua = false;
		} else {
			SequentialAction *seqAction = dynamic_cast<SequentialAction*>(action);
			SimpleAction *simpleAction = seqAction->getSimpleAction();
			if(typeid(*(simpleAction->getGuard())) == typeid(GuardExpression)) {
				expressions.push_back(simpleAction);
			} else if(typeid(*(simpleAction->getGuard())) == typeid(GuardReceive)) {
				receives.push_back(simpleAction);
			} else if(typeid(*(simpleAction->getGuard())) == typeid(GuardTimeout)) {
				timeouts.push_back(simpleAction);
			}
			action = seqAction->getAction();
		}
	}
	this->cFile << "\twhile(1) {\n";
	this->cFile << "\t\tint estado = " << ((receives.size() > 0) ? expressions.size(): 0) << ";\n\n";
	for(unsigned i=0; i < expressions.size(); i++) {
		/*if(expr_i) {
			estado = i;
		} else if(expr_i1) {
			estado = i1;
		}...*/
		GuardExpression *expr = dynamic_cast<GuardExpression*>(expressions[i]->getGuard());
		if(i > 0) {
			this->cFile << " else ";
		}

		this->cFile << "\t\tif(";
		expr->getExpression()->visit(*this);
		this->cFile << ") {\n\t\t\testado = " << i << ";\n\t\t}";
	}

	//switcho
	this->cFile << "\n\t\tswitch(estado) {\n";

	//cases das expressoes
	for(unsigned i=0; i < expressions.size(); i++) {
		this->cFile << "\t\t\tcase " << i << ":\n";
		this->currentTabStop = 4;
		expressions[i]->getStatement()->visit(*this);
		this->cFile << "\t\t\tbreak;\n\n";
	}
	if(receives.size() > 0) {
		//case dos receives
		this->cFile << "\t\t\tcase " << expressions.size() << ":\n";
		this->cFile << "\t\t\t{\n";
		this->cFile << "\t\t\t\tint codes[" << receives.size() << "] = {";
		for(unsigned i=0; i<receives.size(); i++) {
			GuardReceive *receive = dynamic_cast<GuardReceive*>(receives[i]->getGuard());
			int code = this->codeMap[receive->getReceiveStatement()->getMsgId()->getName()];
			if(i > 0)
				this->cFile << ", ";
			this->cFile << code;
		}
		this->cFile << "};\n";

		//tirar essa porcaria pq nao tah pegando do PMSocket certo...
		this->cFile << "\t\t\t\tGenericMessage *msg = pmqueue_dequeue(messageQueue, " << receives.size() << ", codes, NULL);\n";
		//switch dos receives
		this->cFile << "\t\t\t\tswitch(msg->code) {\n";
		//cases dos codigos de msgs
		for(unsigned i=0; i<receives.size(); i++) {
			GuardReceive *receive = dynamic_cast<GuardReceive*>(receives[i]->getGuard());
			this->cFile << "\t\t\t\t\tcase " << this->codeMap[receive->getReceiveStatement()->getMsgId()->getName()] << ":\n";
			this->currentTabStop = 6;
			receives[i]->getStatement()->visit(*this);
			this->cFile << "\t\t\t\t\tbreak;\n";
		}
		//fecha switch dos receives
		this->cFile << "\t\t\t\t}\n";
		//break do case dos receives
		this->cFile << "\t\t\t}\n";
		this->cFile << "\t\t\tbreak;\n\n";
	}
	//fecha switcho
	this->cFile << "\t\t}\n";
	//fecha while
	this->cFile << "\t}\n";

	//fecha thread
	this->cFile << "}\n\n";
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSimpleStatement(SimpleStatement *simpleStatement) {
    VisitorReturn *v = 0;
	/*boiou...*/
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitSkipStatement(SkipStatement *skipStatement) {
    VisitorReturn *v = 0;
	char tabs[20];

	for (unsigned i = 0; i < this->currentTabStop; i++)
		sprintf(tabs, "\t");
	this->cFile << tabs << ";\n";

	return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitStartStatement(StartStatement *startStatement) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitStatement(Statement *statement) {
    VisitorReturn *v = 0;
	//nem sei pra que serve...
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitStopStatement(StopStatement *stopStatement) {
    VisitorReturn *v = 0;
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitStringLiteral(StringLiteral *stringLiteral) {
    VisitorReturn *v = 0;
	this->cFile << stringLiteral->getStr();
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitType(Type *type) {
    VisitorReturn *v = 0;

	switch(type->getTypeCode()) {
		case Type::INTEGER:
		case Type::RANGE:
		case Type::BOOLEAN:
			this->cFile << "int ";
			break;
		case Type::STRING:
			this->cFile << "char ";
			break;
		default:
			break;
	}
    return v;
}

VisitorReturn *CCodeGeneratorVisitor::visitUnaryExpression(UnaryExpression *unaryExpression) {
    VisitorReturn *v = 0;
	string op = unaryExpression->getOp();

	if(op == "&") op = "&&";
	else if (op == "|") op = "||";
	else if (op == "~") op = "!";
	else if (op == "<>") op = "!=";
	else if (op == "=") op = "==";

	this->cFile << op;
	unaryExpression->getExpression()->visit(*this);

    return v;
}
