#ifndef C_CODE_GENERATOR_VISITOR_H
#define C_CODE_GENERATOR_VISITOR_H

#include "visitor.h"
#include "visitorreturn.h"
#include "typecheckervisitor.h"
#include <vector>
#include <fstream>
#include <iostream>
#include <map>
#include <string>
#include <sstream>
using namespace std;

class CCodeGeneratorVisitor : public Visitor {
    private:
        ofstream cFile, hFile;
        const TypeCheckerVisitor::ProcessTableType &symbolTable;
        const TypeCheckerVisitor::BehaviorTableType &behaviorTable;
        char *baseName;
        char *baseDir;
        bool genBehaviorCode;
		bool isAssignment;
		bool isExpression;
		unsigned currentTabStop;
		string currentProcess;
		map<string, int> codeMap;
		vector<Ast*> pilha;
		map<string, string> temps;

		void generateMessageFile();
		void generateGenericMessageHeader(VisitorReturn *v);
		void addBehaviors();
		void generateIPFile(vector<string> &names);
		void initializeArrays(vector<ArrayType*> &ats,
							  vector<string> &arrayNames);

    public:
        CCodeGeneratorVisitor(const TypeCheckerVisitor::ProcessTableType &symbolTable,
                              const TypeCheckerVisitor::BehaviorTableType &behaviorTable,
                              char *baseName, char *baseDir="./", bool genBehaviorCode = false)
            : symbolTable(symbolTable), behaviorTable(behaviorTable), baseName(baseName),
			  baseDir(baseDir), genBehaviorCode(genBehaviorCode), isAssignment(false),
			  isExpression(false), currentTabStop(0), currentProcess("") {}

        virtual ~CCodeGeneratorVisitor() {
            if (this->cFile.is_open())
                this->cFile.close();

            if (this->hFile.is_open())
                this->hFile.close();
        }

        VisitorReturn *visitAction(Action *action);
        VisitorReturn *visitActStatement(ActStatement *actStatement);
        VisitorReturn *visitArrayReference(ArrayReference *arrayReference);
        VisitorReturn *visitArrayType(ArrayType *arrayType);
        VisitorReturn *visitAssignStatement(AssignStatement *assignStatement);
        VisitorReturn *visitAst(Ast *ast);
        VisitorReturn *visitBehavior(Behavior *behavior);
        VisitorReturn *visitBehaviorStatement(BehaviorStatement *behaviorStatement);
        VisitorReturn *visitBinaryExpression(BinaryExpression *binaryExpression);
        VisitorReturn *visitDeclaration(Declaration *declaration);
        VisitorReturn *visitDoStatement(DoStatement *doStatement);
        VisitorReturn *visitExpression(Expression *expression);
        VisitorReturn *visitExpressionList(ExpressionList *expressionList);
        VisitorReturn *visitFieldDeclaration(FieldDeclaration *fieldDeclaration);
        VisitorReturn *visitFieldReference(FieldReference *fieldReference);
        VisitorReturn *visitFieldType(FieldType *fieldType);
        VisitorReturn *visitGuard(Guard *guard);
        VisitorReturn *visitGuardedStatement(GuardedStatement *guardedStatement);
        VisitorReturn *visitGuardExpression(GuardExpression *guardExpression);
        VisitorReturn *visitGuardReceive(GuardReceive *guardReceive);
        VisitorReturn *visitGuardTimeout(GuardTimeout *guardTimeout);
        VisitorReturn *visitIdentifier(Identifier *identifier);
        VisitorReturn *visitIdentifierList(IdentifierList *identifierList);
        VisitorReturn *visitIfStatement(IfStatement *ifStatement);
        VisitorReturn *visitImport(Import *import);
        VisitorReturn *visitIntegerLiteral(IntegerLiteral *integerLiteral);
        VisitorReturn *visitLeftSide(LeftSide *leftSide);
        VisitorReturn *visitLeftSideList(LeftSideList *leftSideList);
        VisitorReturn *visitMessage(Message *message);
        VisitorReturn *visitMultipleArrayReference(MultipleArrayReference *multipleArrayReference);
        VisitorReturn *visitProcess(Process *process);
        VisitorReturn *visitProgram(Program *program);
        VisitorReturn *visitRangeType(RangeType *rangeType);
        VisitorReturn *visitReceiveStatement(ReceiveStatement *receiveStatement);
        VisitorReturn *visitSendStatement(SendStatement *sendStatement);
        VisitorReturn *visitSequentialAction(SequentialAction *sequentialAction);
        VisitorReturn *visitSequentialBehavior(SequentialBehavior *sequentialBehavior);
        VisitorReturn *visitSequentialDeclaration(SequentialDeclaration *sequentialDeclaration);
        VisitorReturn *visitSequentialExpressionList(SequentialExpressionList *sequentialExpressionList);
        VisitorReturn *visitSequentialFieldDeclaration(SequentialFieldDeclaration *sequentialFieldDeclaration);
        VisitorReturn *visitSequentialGuardedStatement(SequentialGuardedStatement *sequentialGuardedStatement);
        VisitorReturn *visitSequentialIdentifierList(SequentialIdentifierList *sequentialIdentifierList);
        VisitorReturn *visitSequentialImport(SequentialImport *sequentialImport);
        VisitorReturn *visitSequentialLeftSideList(SequentialLeftSideList *sequentialLeftSideList);
        VisitorReturn *visitSequentialMessage(SequentialMessage *sequentialMessage);
        VisitorReturn *visitSequentialProcess(SequentialProcess *sequentialProcess);
        VisitorReturn *visitSequentialStatement(SequentialStatement *sequentialStatement);
        VisitorReturn *visitSimpleAction(SimpleAction *simpleAction);
        VisitorReturn *visitSimpleArrayReference(SimpleArrayReference *simpleArrayReference);
        VisitorReturn *visitSimpleBehavior(SimpleBehavior *simpleBehavior);
        VisitorReturn *visitSimpleDeclaration(SimpleDeclaration *simpleDeclaration);
        VisitorReturn *visitSimpleExpressionList(SimpleExpressionList *simpleExpressionList);
        VisitorReturn *visitSimpleFieldDeclaration(SimpleFieldDeclaration *simpleFieldDeclaration);
        VisitorReturn *visitSimpleGuardedStatement(SimpleGuardedStatement *simpleGuardedStatement);
        VisitorReturn *visitSimpleIdentifierList(SimpleIdentifierList *simpleIdentifierList);
        VisitorReturn *visitSimpleImport(SimpleImport *simpleImport);
        VisitorReturn *visitSimpleLeftSideList(SimpleLeftSideList *simpleLeftSideList);
        VisitorReturn *visitSimpleMessage(SimpleMessage *simpleMessage);
        VisitorReturn *visitSimpleProcess(SimpleProcess *simpleProcess);
        VisitorReturn *visitSimpleStatement(SimpleStatement *simpleStatement);
        VisitorReturn *visitSkipStatement(SkipStatement *skipStatement);
        VisitorReturn *visitStartStatement(StartStatement *startStatement);
        VisitorReturn *visitStatement(Statement *statement);
        VisitorReturn *visitStopStatement(StopStatement *stopStatement);
        VisitorReturn *visitStringLiteral(StringLiteral *stringLiteral);
        VisitorReturn *visitType(Type *type);
        VisitorReturn *visitUnaryExpression(UnaryExpression *unaryExpression);
};

#endif
