#ifndef PROGRAM_H
#define PROGRAM_H

#include "ast.h"
#include "import.h"
#include "behavior.h"
#include "message.h"
#include "process.h"

class Program : public Ast {
	private:
		Import *import;
		Behavior *behavior;
		Message *message;
		Process *process;

	public:
		Program(SrcPosition *position, Import *import, Behavior *behavior, Message *message, Process *process)
				: Ast(position), import(import), behavior(behavior), message(message), process(process) {}

		Program(const Program& program) : Ast(program.position) {
			*(this) = program;
		}

		virtual ~Program() {
			delete this->import;
			delete this->behavior;
			delete this->message;
			delete this->process;
		}

		Import *getImport() {
			return this->import;
		}

		Behavior *getBehavior() {
			return this->behavior;
		}

		Message *getMessage() {
			return this->message;
		}

		Process *getProcess() {
			return this->process;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitProgram(this);
		}

		virtual Ast& clone() const {
			return *(new Program(new SrcPosition(*(this->position)),
					 this->import ? dynamic_cast<Import*>(&(this->import->clone())) : 0,
					 this->behavior ? dynamic_cast<Behavior*>(&(this->behavior->clone())) : 0,
					 this->message ? dynamic_cast<Message*>(&(this->message->clone())) : 0,
					 this->process ? dynamic_cast<Process*>(&(this->process->clone())) : 0));
		}
};

#endif
