#ifndef RANGE_TYPE_H
#define RANGE_TYPE_H

#include "ast.h"

class RangeType : public Type {
	private:
		int start;
		int end;
		
	public:
		RangeType(SrcPosition *position, types typeCode, int start, int end)
				: Type(position, typeCode), start(start), end(end) {}

		RangeType(const RangeType &rangeType) : Type(rangeType.position, rangeType.typeCode) {
			*(this) = rangeType;
		}
		
		virtual ~RangeType() {
		}

		virtual Ast& operator = (const Ast& ast) {
			const RangeType& rangeType = dynamic_cast<const RangeType&>(ast); 
			Type::operator=(ast);
			this->start = rangeType.start;
			this->end = rangeType.end;

			return (*this);
		}

		int getStart() const {
			return this->start;
		}

		int getEnd() const {
			return this->end;
		}
		
		virtual bool operator == (const Ast& ast) const {
			const RangeType& rangeType = dynamic_cast<const RangeType&>(ast);
			return Type::operator==(ast) && (this->start == rangeType.start) && (this->end == rangeType.end);
		}

		virtual Ast& clone() const {
			return *(new RangeType(new SrcPosition(*(this->position)), this->typeCode, this->start, this->end));
		}

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitRangeType(this);
		}
};

#endif
