#ifndef SEQUENTIAL_EXPRESSION_LIST_H
#define SEQUENTIAL_EXPRESSION_LIST_H

#include "expressionlist.h"
#include "simpleexpressionlist.h"

class SequentialExpressionList : public ExpressionList {
	private:
		ExpressionList *expressionList;
		SimpleExpressionList *simpleExpressionList;


	public:
		SequentialExpressionList(SrcPosition *position, ExpressionList *expressionList, SimpleExpressionList *simpleExpressionList)
			: ExpressionList(position), expressionList(expressionList), simpleExpressionList(simpleExpressionList) {}

		SequentialExpressionList(const SequentialExpressionList& sequentialExpressionList) : ExpressionList(sequentialExpressionList.position) {
			*(this) = sequentialExpressionList;
		}

		virtual ~SequentialExpressionList() {
			delete this->expressionList;
			delete this->simpleExpressionList;
		}

		ExpressionList *getExpressionList() {
			return this->expressionList;
		}

		SimpleExpressionList *getSimpleExpressionList() {
			return this->simpleExpressionList;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSequentialExpressionList(this);
		}

		virtual Ast& clone() const {
			return *(new SequentialExpressionList(new SrcPosition(*(this->position)),
					 this->expressionList ? dynamic_cast<ExpressionList*>(&(this->expressionList->clone())) : 0,
					 this->simpleExpressionList ? dynamic_cast<SimpleExpressionList*>(&(this->simpleExpressionList->clone())) : 0));
		}
};

#endif
