#ifndef SEQUENTIAL_GUARDED_STATEMENT_H
#define SEQUENTIAL_GUARDED_STATEMENT_H

#include "guardedstatement.h"
#include "simpleguardedstatement.h"

class SequentialGuardedStatement : public GuardedStatement {
	private:
		GuardedStatement *guardedStatement;
		SimpleGuardedStatement *simpleGuardedStatement;


	public:
		SequentialGuardedStatement(SrcPosition *position, GuardedStatement *guardedStatement, SimpleGuardedStatement *simpleGuardedStatement)
			: GuardedStatement(position), guardedStatement(guardedStatement), simpleGuardedStatement(simpleGuardedStatement) {}

		SequentialGuardedStatement(const SequentialGuardedStatement& sequentialGuardedStatement) : GuardedStatement(sequentialGuardedStatement.position) {
			*(this) = sequentialGuardedStatement;
		}

		virtual ~SequentialGuardedStatement() {
			delete this->guardedStatement;
			delete this->simpleGuardedStatement;
		}

		GuardedStatement *getGuardedStatement() {
			return this->guardedStatement;
		}

		SimpleGuardedStatement *getSimpleGuardedStatement() {
			return this->simpleGuardedStatement;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSequentialGuardedStatement(this);
		}

		virtual Ast& clone() const {
			return *(new SequentialGuardedStatement(new SrcPosition(*(this->position)),
					 this->guardedStatement ? dynamic_cast<GuardedStatement*>(&(this->guardedStatement->clone())) : 0,
					 this->simpleGuardedStatement ? dynamic_cast<SimpleGuardedStatement*>(&(this->simpleGuardedStatement->clone())) : 0));
		}
};

#endif
