#ifndef SEQUENTIAL_IMPORT_H
#define SEQUENTIAL_IMPORT_H

#include "import.h"
#include "simpleimport.h"

class SequentialImport : public Import {
	private:
		Import *import;
		SimpleImport *simpleImport;


	public:
		SequentialImport(SrcPosition *position, Import *import, SimpleImport *simpleImport)
			: Import(position), import(import), simpleImport(simpleImport) {}

		SequentialImport(const SequentialImport& sequentialImport) : Import(sequentialImport.position) {
			*(this) = sequentialImport;
		}

		virtual ~SequentialImport() {
			delete this->import;
			delete this->simpleImport;
		}

		Import *getImport() {
			return this->import;
		}

		SimpleImport *getSimpleImport() {
			return this->simpleImport;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSequentialImport(this);
		}

		virtual Ast& clone() const {
			return *(new SequentialImport(new SrcPosition(*(this->position)),
					 this->import ? dynamic_cast<Import*>(&(this->import->clone())) : 0,
					 this->simpleImport ? dynamic_cast<SimpleImport*>(&(this->simpleImport->clone())) : 0));
		}
};

#endif
