#ifndef SIMPLE_ACTION_H
#define SIMPLE_ACTION_H

#include "action.h"
#include "guard.h"
#include "statement.h"

class SimpleAction : public Action {
	private:
		Guard *guard;
		Statement *statement;

	public:
		SimpleAction(SrcPosition *position, Guard *guard, Statement *statement)
			: Action(position), guard(guard), statement(statement) {}

		SimpleAction(const SimpleAction& simpleAction) : Action(simpleAction.position) {
			*(this) = simpleAction;
		}

		virtual ~SimpleAction() {
			delete this->guard;
			delete this->statement;
		}

		Guard *getGuard() {
			return this->guard;
		}

		Statement *getStatement() {
			return this->statement;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleAction(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleAction(new SrcPosition(*(this->position)),
					 this->guard ? dynamic_cast<Guard*>(&(this->guard->clone())) : 0,
					 this->statement ? dynamic_cast<Statement*>(&(this->statement->clone())) : 0));
		}
};

#endif
