#ifndef SIMPLE_ARRAY_REFERENCE_H
#define SIMPLE_ARRAY_REFERENCE_H

#include "arrayreference.h"
#include "identifier.h"
#include "expression.h"

class SimpleArrayReference : public ArrayReference {
	private:
		Identifier *identifier;
		Expression *expression;

	public:
		SimpleArrayReference(SrcPosition *position, Identifier *identifier, Expression *expression)
			: Ast(position), ArrayReference(position), identifier(identifier), expression(expression) {}

		SimpleArrayReference(const SimpleArrayReference& simpleArrayReference) : Ast(position), ArrayReference(simpleArrayReference.position) {
			*(this) = simpleArrayReference;
		}

		virtual ~SimpleArrayReference() {
			delete this->identifier;
			delete this->expression;
		}

		Identifier *getIdentifier() {
			return this->identifier;
		}

		Expression *getExpression() {
			return this->expression;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleArrayReference(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleArrayReference(new SrcPosition(*(this->position)),
					 this->identifier ? dynamic_cast<Identifier*>(&(this->identifier->clone())) : 0,
					 this->expression ? dynamic_cast<Expression*>(&(this->expression->clone())) : 0));
		}
};

#endif
