#ifndef SIMPLE_DECLARATION_H
#define SIMPLE_DECLARATION_H

#include "declaration.h"
#include "identifierlist.h"
#include "type.h"
#include "expression.h"

class SimpleDeclaration : public Declaration {
	private:
		IdentifierList *identifierList;
		Type *type;
		Expression *expression;
		bool constant;

	public:
		SimpleDeclaration(SrcPosition *position, IdentifierList *identifierList, Type *type, 
						  Expression *expression, bool constant)
			: Declaration(position), identifierList(identifierList), type(type), 
			  expression(expression), constant(constant) {}

		SimpleDeclaration(const SimpleDeclaration& simpleDeclaration) : Declaration(simpleDeclaration.position) {
			*(this) = simpleDeclaration;
		}

		virtual ~SimpleDeclaration() {
			delete this->identifierList;
			delete this->type;
			delete this->expression;
		}

		IdentifierList *getIdentifierList() {
			return this->identifierList;
		}

		Type *getType() {
			return this->type;
		}

		Expression *getExpression() {
			return this->expression;
		}

		bool isConstant() const {
			return this->constant;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleDeclaration(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleDeclaration(new SrcPosition(*(this->position)),
					 this->identifierList ? dynamic_cast<IdentifierList*>(&(this->identifierList->clone())) : 0,
					 this->type ? dynamic_cast<Type*>(&(this->type->clone())) : 0,
					 this->expression ? dynamic_cast<Expression*>(&(this->expression->clone())) : 0,
					 this->constant));
		}
};

#endif
