#ifndef SIMPLE_PROCESS_H
#define SIMPLE_PROCESS_H

#include "process.h"
#include "identifier.h"
#include "declaration.h"
#include "action.h"

class SimpleProcess : public Process {
	private:
		Identifier *identifier;
		Declaration *declaration;
		Action *action;
		bool server;
		bool mainProcess;

	public:
		SimpleProcess(SrcPosition *position, Identifier *identifier, Declaration *declaration, Action *action, bool server, bool mainProcess)
			: Process(position), identifier(identifier), declaration(declaration), action(action), server(server), mainProcess(mainProcess) {}

		SimpleProcess(const SimpleProcess& simpleProcess) : Process(simpleProcess.position) {
			*(this) = simpleProcess;
		}

		virtual ~SimpleProcess() {
			delete this->identifier;
			delete this->declaration;
			delete this->action;
		}

		Identifier *getIdentifier() {
			return this->identifier;
		}

		Declaration *getDeclaration() {
			return this->declaration;
		}

		Action *getAction() {
			return this->action;
		}

		bool isServer() const {
			return this->server;
		}

		bool isMainProcess() const {
			return this->mainProcess;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleProcess(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleProcess(new SrcPosition(*(this->position)),
					 this->identifier ? dynamic_cast<Identifier*>(&(this->identifier->clone())) : 0,
					 this->declaration ? dynamic_cast<Declaration*>(&(this->declaration->clone())) : 0,
					 this->action ? dynamic_cast<Action*>(&(this->action->clone())) : 0, this->server,
					 this->mainProcess));
		}
};

#endif
