#ifndef TYPE_CHECKER_VISITOR_H
#define TYPE_CHECKER_VISITOR_H
#define GLOBAL "&GLOBAL&"

#include "visitor.h"
#include <map>
#include <vector>
#include <string>

using namespace std;

class TypeCheckerVisitor : public Visitor {
	private:
		map<string, map<string, Type*> > symbolTable;
		bool insert;
		bool forgetIt;
		string currentProcess;
		vector<Ast*> stack;
		bool inBehaviorStatement;
		vector<int> behaviorTypes;
		multimap<string, vector<int>* > behaviorTable;

	public:
		typedef map<string, map<string, Type*> > ProcessTableType;
		typedef map<string, Type*> SymbolTableType;
        typedef multimap<string, vector<int>* > BehaviorTableType;

		virtual ~TypeCheckerVisitor() {}
		const ProcessTableType &getSymbolTable() const {
			return this->symbolTable;
		}
		const BehaviorTableType &getBehaviorTable() const {
			return this->behaviorTable;
		}
		VisitorReturn *visitAction(Action *action);
		VisitorReturn *visitActStatement(ActStatement *actStatement);
		VisitorReturn *visitArrayReference(ArrayReference *arrayReference);
		VisitorReturn *visitArrayType(ArrayType *arrayType);
		VisitorReturn *visitAssignStatement(AssignStatement *assignStatement);
		VisitorReturn *visitAst(Ast *ast);
		VisitorReturn *visitBehavior(Behavior *behavior);
		VisitorReturn *visitBehaviorStatement(BehaviorStatement *behaviorStatement);
		VisitorReturn *visitBinaryExpression(BinaryExpression *binaryExpression);
		VisitorReturn *visitDeclaration(Declaration *declaration);
		VisitorReturn *visitDoStatement(DoStatement *doStatement);
		VisitorReturn *visitExpression(Expression *expression);
		VisitorReturn *visitExpressionList(ExpressionList *expressionList);
		VisitorReturn *visitFieldDeclaration(FieldDeclaration *fieldDeclaration);
		VisitorReturn *visitFieldReference(FieldReference *fieldReference);
		VisitorReturn *visitFieldType(FieldType *fieldType);
		VisitorReturn *visitGuard(Guard *guard);
		VisitorReturn *visitGuardedStatement(GuardedStatement *guardedStatement);
		VisitorReturn *visitGuardExpression(GuardExpression *guardExpression);
		VisitorReturn *visitGuardReceive(GuardReceive *guardReceive);
		VisitorReturn *visitGuardTimeout(GuardTimeout *guardTimeout);
		VisitorReturn *visitIdentifier(Identifier *identifier);
		VisitorReturn *visitIdentifierList(IdentifierList *identifierList);
		VisitorReturn *visitIfStatement(IfStatement *ifStatement);
		VisitorReturn *visitImport(Import *import);
		VisitorReturn *visitLeftSide(LeftSide *leftSide);
		VisitorReturn *visitLeftSideList(LeftSideList *leftSideList);
		VisitorReturn *visitMessage(Message *message);
		VisitorReturn *visitMultipleArrayReference(MultipleArrayReference *multipleArrayReference);
		VisitorReturn *visitProcess(Process *process);
		VisitorReturn *visitProgram(Program *program);
		VisitorReturn *visitRangeType(RangeType *rangeType);
		VisitorReturn *visitReceiveStatement(ReceiveStatement *receiveStatement);
		VisitorReturn *visitSendStatement(SendStatement *sendStatement);
		VisitorReturn *visitSequentialAction(SequentialAction *sequentialAction);
		VisitorReturn *visitSequentialBehavior(SequentialBehavior *sequentialBehavior);
		VisitorReturn *visitSequentialDeclaration(SequentialDeclaration *sequentialDeclaration);
		VisitorReturn *visitSequentialExpressionList(SequentialExpressionList *sequentialExpressionList);
		VisitorReturn *visitSequentialFieldDeclaration(SequentialFieldDeclaration *sequentialFieldDeclaration);
		VisitorReturn *visitSequentialGuardedStatement(SequentialGuardedStatement *sequentialGuardedStatement);
		VisitorReturn *visitSequentialIdentifierList(SequentialIdentifierList *sequentialIdentifierList);
		VisitorReturn *visitSequentialImport(SequentialImport *sequentialImport);
		VisitorReturn *visitSequentialLeftSideList(SequentialLeftSideList *sequentialLeftSideList);
		VisitorReturn *visitSequentialMessage(SequentialMessage *sequentialMessage);
		VisitorReturn *visitSequentialProcess(SequentialProcess *sequentialProcess);
		VisitorReturn *visitSequentialStatement(SequentialStatement *sequentialStatement);
		VisitorReturn *visitSimpleAction(SimpleAction *simpleAction);
		VisitorReturn *visitSimpleArrayReference(SimpleArrayReference *simpleArrayReference);
		VisitorReturn *visitSimpleBehavior(SimpleBehavior *simpleBehavior);
		VisitorReturn *visitSimpleDeclaration(SimpleDeclaration *simpleDeclaration);
		VisitorReturn *visitSimpleExpressionList(SimpleExpressionList *simpleExpressionList);
		VisitorReturn *visitSimpleFieldDeclaration(SimpleFieldDeclaration *simpleFieldDeclaration);
		VisitorReturn *visitSimpleGuardedStatement(SimpleGuardedStatement *simpleGuardedStatement);
		VisitorReturn *visitSimpleIdentifierList(SimpleIdentifierList *simpleIdentifierList);
		VisitorReturn *visitSimpleImport(SimpleImport *simpleImport);
		VisitorReturn *visitSimpleLeftSideList(SimpleLeftSideList *simpleLeftSideList);
		VisitorReturn *visitSimpleMessage(SimpleMessage *simpleMessage);
		VisitorReturn *visitSimpleProcess(SimpleProcess *simpleProcess);
		VisitorReturn *visitSimpleStatement(SimpleStatement *simpleStatement);
		VisitorReturn *visitSkipStatement(SkipStatement *skipStatement);
		VisitorReturn *visitStartStatement(StartStatement *startStatement);
		VisitorReturn *visitStatement(Statement *statement);
		VisitorReturn *visitStopStatement(StopStatement *stopStatement);
		VisitorReturn *visitStringLiteral(StringLiteral *stringLiteral);
		VisitorReturn *visitIntegerLiteral(IntegerLiteral *integerLiteral);
		VisitorReturn *visitType(Type *type);
		VisitorReturn *visitUnaryExpression(UnaryExpression *unaryExpression);
};

#endif
